# -*- coding: utf-8 -*-

import datetime
import json

import click
import dateutil
import dateutil.parser

from em_celery import logger, init_db
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.order_util import OrderUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.store.models import *


@click.command('Upload Elim Forwardings')
@click.option('-s', '--store_code', required=True, type=str, help='Store code.')
def upload_forwardings(store_code):
  init_db()

  store = StoreUtil.get_store_by_code(store_code)
  if not store:
    logger.warning('[StoreNotFound] %s', store_code)
    return

  service_account_data = json.loads(store.service_account_data)
  uploader = ForwardingUploader(service_account_data)
  uploader.run()


class ForwardingUploader():
  def __init__(self, service_account_data):
    self.vendor_id = 35
    self.service_account_data = service_account_data
    self.sheet_api = SheetAPI(service_account_data)
    self.spreadsheets = [
      {'sheet_id': '1ilJhdGRS6mpG8bIRPGtU8C6mA8UpVQOIsWZ_dSwFY2Q', 'sheet_name': 'Shipping'}, # ELIM UK
      {'sheet_id': '1xJ_sJ2SIzYh-dLHkIunp7b1tDOpStxrq1_6b39AKH0I', 'sheet_name': 'Forwarding'}, # ELIM DE
    ]

  def run(self):
    now = datetime.datetime.now()
    for spreadsheet_info in self.spreadsheets:
      forwardings = {}

      sheet_id = spreadsheet_info['sheet_id']
      sheet_name = spreadsheet_info['sheet_name']
      spreadsheet = None
      worksheet = None
      try:
        spreadsheet = self.sheet_api.get_spreadsheet(sheet_id)
        if not spreadsheet:
          logger.error('[SpreadSheetNotFound] %s', sheet_id)
          continue

        worksheet = spreadsheet.worksheet(sheet_name)
        if not worksheet:
          logger.error('[WorkSheetNotFound] SpreadSheet: %s, WorkSheet: %s', sheet_id, sheet_name)
          continue
      except Exception as e:
        logger.exception(e)
        continue

      if not spreadsheet or not worksheet:
        continue

      vals = self.sheet_api.get_worksheet_vals(worksheet)
      headers = []
      for row in vals:
        if not headers:
          if 'tracking' not in row and 'Tracking' not in row and 'Tracking Number' not in row:
            continue

          for col in row:
            col_lower = col.lower()
            col_name = col_lower
            if col_lower.find('carrier') != -1:
              col_name = 'carrier'
            elif col_lower.find('tracking') != -1:
              col_name = 'tracking_number'
            elif col_lower.find('date') != -1:
              col_name = 'shipped_at'
            elif col_lower.find('cost') != -1 or col_lower.find('amount') != -1:
              col_name = 'amount'

            headers.append(col_name)

          continue

        empty = True
        for col in row:
          if col:
            empty = False
            break

        if empty:
          continue

        record = {}
        for idx, header in enumerate(headers):
          try:
            record[header] = row[idx].strip()
          except Exception as e:
            logger.exception(e)

        if not record['amount'] or not record['shipped_at'] or not record['carrier'] or not record['tracking_number']:
          continue

        cost = None
        try:
          cost = round(float(record['amount']), 2)
        except:
          pass

        if not cost:
          continue

        try:
          shipped_at = dateutil.parser.parse(record['shipped_at'])
          if not shipped_at:
            continue
        except Exception as e:
          continue

        forwardings[record['tracking_number']] = {
          'vendor_id': self.vendor_id,
          'carrier': record['carrier'],
          'tracking_number': record['tracking_number'],
          'amount': cost,
          'shipped_at': shipped_at,
          'note': record.get('note', None),
          'created_at': shipped_at,
          'updated_at': shipped_at
        }

      if not forwardings:
        continue

      for seller_forwarding in SellerForwarding.select(SellerForwarding.tracking_number).where(SellerForwarding.vendor_id == self.vendor_id).execute():
        forwardings.pop(seller_forwarding.tracking_number, None)

      if not forwardings:
        continue

      logger.info('[SellerForwardings] Count: %s, Amount: %s', len(forwardings), sum([forwarding['amount'] for forwarding in forwardings.values()]))

      SellerForwarding.insert_many(list(forwardings.values())).execute()


if __name__ == "__main__":
  upload_forwardings()
