# -*- coding: utf-8 -*-

import datetime
import json

import click

from em_celery import logger, init_db
from em_tasks.utils.price_calculator import PriceCalculator
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.order_util import OrderUtil
from em_tasks.utils.pipelines.price_calculator import PriceCalculatorPipeline
from em_tasks.utils.pipelines.sku_updater import SkuUpdaterPipeline
from em_tasks.utils.pipelines.image_url_corrector import ImageUrlCorrectorPipeline
from em_tasks.utils.pipelines.upc_validator import UpcValidatorPipeline
from em_tasks.store.models import *

from em_celery import logger, get_config, init_db, get_product_service


@click.command('Format products to Spree style')
@click.option('-s', '--source', required=True, type=str, help='Product source.')
@click.option('-sc', '--source_code', required=True, type=str, help='Product source code.')
@click.option('-sp', '--sku_prefix', type=str, default='', help='Current product sku prefix to replace.')
@click.option('-o', '--output_path', required=True, type=str)
@click.argument('products_path')
def format_products(products_path, output_path, source, source_code, sku_prefix=''):
  init_db()

  if not products_path or not output_path:
    return

  products_path = os.path.abspath(os.path.expanduser(products_path))
  if not os.path.isfile(products_path):
    return

  output_path = os.path.abspath(os.path.expanduser(output_path))
  output_dir = os.path.dirname(output_path)
  if not os.path.isdir(output_dir):
    os.makedirs(output_dir)

  price_rules = {
    'roi': 0.3,
    'ad_cost': 5,
    'transfer_cost': 12
  }

  cfg = get_config()
  price_rules_cfg = cfg.get('price.rules.{}'.format(source.lower()), dict())
  for k, v in price_rules_cfg.items():
    price_rules[k] = round(float(v), 2)
  price_calculator = PriceCalculator(price_rules)

  price_calculator_pipeline = PriceCalculatorPipeline(price_calculator)
  sku_updater_pipeline = SkuUpdaterPipeline(sku_prefix, source_code)
  image_url_corrector_pipeline = ImageUrlCorrectorPipeline()
  upc_validator_pipeline = UpcValidatorPipeline()
  pipelines = [price_calculator_pipeline, sku_updater_pipeline, image_url_corrector_pipeline, upc_validator_pipeline]

  with open(output_path, 'w+', encoding='utf-8', errors='ignore') as output_fh:
    with open(products_path, encoding='utf-8', errors='ignore') as fh:
      for line in fh:
        s = line.strip()
        if not s:
          continue

        prod = None
        try:
          prod = json.loads(s)
        except Exception as e:
          logger.exception(e)

        if not prod:
          continue

        prod['source'] = source
        product = prod
        for pipeline in pipelines:
          product = pipeline.execute(product)
          if not product:
            break

        if not product or not product.get('price', 0):
          continue

        output_fh.write(json.dumps(product, ensure_ascii=False))
        output_fh.write("\n")


if __name__ == "__main__":
  format_products()
